import 'dart:io';

import 'package:cached_network_image/cached_network_image.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:tencent_im_sdk_plugin/models/v2_tim_message.dart';

import 'package:tim_ui_kit/ui/utils/color.dart';
import 'package:tim_ui_kit/ui/utils/message.dart';
import 'package:tim_ui_kit/ui/utils/route.dart';
import 'package:tim_ui_kit/ui/utils/tui_theme.dart';
import 'package:tim_ui_kit/ui/widgets/video_screen.dart';
import 'package:transparent_image/transparent_image.dart';

import '../../../../i18n/i18n_utils.dart';
import '../tim_uikit_chat.dart';

class TIMUIKitVideoElem extends StatefulWidget {
  const TIMUIKitVideoElem(this.message, {Key? key}) : super(key: key);
  final V2TimMessage message;

  @override
  State<StatefulWidget> createState() => _TIMUIKitVideoElemState();
}

class _TIMUIKitVideoElemState extends State<TIMUIKitVideoElem> {
  Widget errorDisplay(TUITheme? theme) {
    final I18nUtils ttBuild = I18nUtils(context);
    return Container(
      decoration: BoxDecoration(
          border: Border.all(
        width: 1,
        color: Colors.black12,
      )),
      height: 100,
      child: Center(
        child: Row(
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Icon(
              Icons.warning_amber_outlined,
              color: theme?.cautionColor,
              size: 16,
            ),
            Text(
              ttBuild.imt("视频加载失败"),
              style: TextStyle(color: theme?.cautionColor),
            ),
          ],
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final theme = SharedThemeWidget.of(context)?.theme;

    return GestureDetector(
      onTap: () {
        Navigator.of(context)
            .push(FadeRoute(page: VideoScreen(message: widget.message)));
      },
      child: Container(
        // constraints: BoxConstraints(
        //     maxWidth: adaptWidth["width"], maxHeight: adaptWidth["height"]),
        decoration: BoxDecoration(
          border: Border.all(
            color: theme?.weakDividerColor ?? CommonColor.weakDividerColor,
            width: 1,
            style: BorderStyle.solid,
          ),
          borderRadius: const BorderRadius.all(Radius.circular(10)),
        ),
        child: ClipRRect(
          borderRadius: const BorderRadius.all(Radius.circular(5)),
          child: LayoutBuilder(
              builder: (BuildContext context, BoxConstraints constraints) {
            return ConstrainedBox(
                constraints: BoxConstraints(
                    maxWidth: constraints.maxWidth * 0.4, minWidth: 0),
                child: Stack(
                  children: <Widget>[
                    Positioned(
                        child: widget.message.videoElem!.snapshotUrl == null
                            ? ConstrainedBox(
                                constraints: BoxConstraints(
                                    maxHeight: 420.h, //宽度尽可能大
                                    maxWidth: 320.w //最小高度为50像素
                                    ),
                                child: Image.file(
                                    File(widget
                                        .message.videoElem!.snapshotPath!),
                                    fit: BoxFit.fitWidth),
                              )
                            : CachedNetworkImage(
                                placeholder: (context, url) => Image(
                                    image: MemoryImage(kTransparentImage)),
                                cacheKey: widget.message.videoElem!.UUID,
                                fit: BoxFit.fitWidth,
                                imageUrl:
                                    widget.message.videoElem!.snapshotUrl!,
                                errorWidget: (context, url, error) =>
                                    errorDisplay(theme),
                              )),
                    Positioned.fill(
                      // alignment: Alignment.center,
                      child: Center(
                          child: Image.asset('images/play.png',
                              package: 'tim_ui_kit', height: 64)),
                    ),
                    Positioned(
                        right: 10,
                        bottom: 10,
                        child: Text(
                            MessageUtils.formatVideoTime(
                                    widget.message.videoElem?.duration ?? 0)
                                .toString(),
                            style: const TextStyle(
                                color: Colors.white, fontSize: 12))),
                  ],
                ));
          }),
        ),
      ),
    );
  }
}
